@@ -10,6 +10,7 @@ hierarchy.
 module Hledger.Data.AccountName
 where
 import Data.List
+import Data.List.Split (splitOn)
 import Data.Tree
 import Test.HUnit
 import Text.Printf
@@ -31,6 +32,16 @@ accountNameFromComponents = concat . intersperse [acctsepchar]
 accountLeafName :: AccountName -> String
 accountLeafName = last . accountNameComponents
 
+-- | Truncate all account name components but the last to two characters.
+accountSummarisedName :: AccountName -> String
+accountSummarisedName a
+  --   length cs > 1 = take 2 (head cs) ++ ":" ++ a'
+  | length cs > 1 = intercalate ":" (map (take 2) $ init cs) ++ ":" ++ a'
+  | otherwise     = a'
+    where
+      cs = accountNameComponents a
+      a' = accountLeafName a
+
 accountNameLevel :: AccountName -> Int
 accountNameLevel "" = 0
 accountNameLevel a = length (filter (==acctsepchar) a) + 1
@@ -100,13 +111,24 @@ nullaccountnametree = Node "root" []
 --     ..:Af:Lu:Sn:Ca:Ch:Cash  ; Abbreviated and elided!
 -- @
 elideAccountName :: Int -> AccountName -> AccountName
-elideAccountName width s =
-    elideLeft width $ accountNameFromComponents $ elideparts width [] $ accountNameComponents s
+elideAccountName width s
+  -- XXX special case for transactions register's multi-account pseudo-names
+  | " (split)" `isSuffixOf` s =
+    let
+      names = splitOn ", " $ take (length s - 8) s
+      widthpername = (max 0 (width - 8 - 2 * (max 1 (length names) - 1))) `div` length names
+    in
+     fitString Nothing (Just width) True False $
+     (++" (split)") $
+     intercalate ", " $
+     [accountNameFromComponents $ elideparts widthpername [] $ accountNameComponents s' | s' <- names]
+  | otherwise =
+    fitString Nothing (Just width) True False $ accountNameFromComponents $ elideparts width [] $ accountNameComponents s
       where
         elideparts :: Int -> [String] -> [String] -> [String]
         elideparts width done ss
-          | length (accountNameFromComponents $ done++ss) <= width = done++ss
-          | length ss > 1 = elideparts width (done++[take 2 $ head ss]) (tail ss)
+          | strWidth (accountNameFromComponents $ done++ss) <= width = done++ss
+          | length ss > 1 = elideparts width (done++[takeWidth 2 $ head ss]) (tail ss)
           | otherwise = done++ss
 
 -- | Keep only the first n components of an account name, where n
