@@ -87,6 +87,7 @@ module Hledger.Data.Amount (
   isReallyZeroMixedAmountCost,
   -- ** rendering
   showMixedAmount,
+  showMixedAmountOneLine,
   showMixedAmountDebug,
   showMixedAmountWithoutPrice,
   showMixedAmountOneLineWithoutPrice,
@@ -118,7 +119,7 @@ import Hledger.Data.Commodity
 import Hledger.Utils
 
 
-deriving instance Show HistoricalPrice
+deriving instance Show MarketPrice
 
 amountstyle = AmountStyle L False 0 (Just '.') Nothing
 
@@ -377,9 +378,11 @@ mixed = normaliseMixedAmount . Mixed
 --
 -- * amounts in the same commodity are combined unless they have different prices or total prices
 --
--- * multiple zero amounts are replaced by just one. If they had the same commodity, it is preserved.
+-- * multiple zero amounts, all with the same non-null commodity, are replaced by just the last of them, preserving the commodity and amount style (all but the last zero amount are discarded)
 --
--- * an empty amount list is replaced with a single commodityless zero
+-- * multiple zero amounts with multiple commodities, or no commodities, are replaced by one commodity-less zero amount
+--
+-- * an empty amount list is replaced by one commodity-less zero amount
 --
 -- * the special "missing" mixed amount remains unchanged
 --
@@ -393,7 +396,7 @@ normaliseHelper squashprices (Mixed as)
   | otherwise            = Mixed nonzeros
   where
     newzero = case filter (/= "") (map acommodity zeros) of
-               [c] -> nullamt{acommodity=c}
+               _:_ -> last zeros
                _   -> nullamt
     (zeros, nonzeros) = partition isReallyZeroAmount $
                         map sumSimilarAmountsUsingFirstPrice $
@@ -517,19 +520,25 @@ isReallyZeroMixedAmountCost = isReallyZeroMixedAmount . costOfMixedAmount
 -- normalising it to one amount per commodity. Assumes amounts have
 -- no or similar prices, otherwise this can show misleading prices.
 showMixedAmount :: MixedAmount -> String
-showMixedAmount = showMixedAmountHelper False
+showMixedAmount = showMixedAmountHelper False False
 
 -- | Like showMixedAmount, but zero amounts are shown with their
 -- commodity if they have one.
 showMixedAmountWithZeroCommodity :: MixedAmount -> String
-showMixedAmountWithZeroCommodity = showMixedAmountHelper True
+showMixedAmountWithZeroCommodity = showMixedAmountHelper True False
 
-showMixedAmountHelper :: Bool -> MixedAmount -> String
-showMixedAmountHelper showzerocommodity m =
-  vConcatRightAligned $ map showw $ amounts $ normaliseMixedAmountSquashPricesForDisplay m
+-- | Get the one-line string representation of a mixed amount.
+showMixedAmountOneLine :: MixedAmount -> String
+showMixedAmountOneLine = showMixedAmountHelper False True
+
+showMixedAmountHelper :: Bool -> Bool -> MixedAmount -> String
+showMixedAmountHelper showzerocommodity useoneline m =
+  join $ map showamt $ amounts $ normaliseMixedAmountSquashPricesForDisplay m
   where
-    showw | showzerocommodity = showAmountWithZeroCommodity
-          | otherwise         = showAmount
+    join | useoneline = intercalate ", "
+         | otherwise  = vConcatRightAligned
+    showamt | showzerocommodity = showAmountWithZeroCommodity
+            | otherwise         = showAmount
 
 -- | Compact labelled trace of a mixed amount, for debugging.
 ltraceamount :: String -> MixedAmount -> MixedAmount
@@ -622,13 +631,13 @@ tests_Hledger_Data_Amount = TestList $
 
   -- MixedAmount
 
-  ,"adding mixed amounts, preserving minimum precision and a single commodity on zero" ~: do
+  ,"adding mixed amounts to zero, the commodity and amount style are preserved" ~: do
     (sum $ map (Mixed . (:[]))
              [usd 1.25
-             ,usd (-1) `withPrecision` 0
+             ,usd (-1) `withPrecision` 3
              ,usd (-0.25)
              ])
-      `is` Mixed [usd 0 `withPrecision` 0]
+      `is` Mixed [usd 0 `withPrecision` 3]
 
   ,"adding mixed amounts with total prices" ~: do
     (sum $ map (Mixed . (:[]))
