@@ -9,7 +9,7 @@ other data format (see "Hledger.Read").
 
 module Hledger.Data.Journal (
   -- * Parsing helpers
-  addHistoricalPrice,
+  addMarketPrice,
   addModifierTransaction,
   addPeriodicTransaction,
   addTimeLogEntry,
@@ -35,6 +35,9 @@ module Hledger.Data.Journal (
   journalDescriptions,
   journalFilePath,
   journalFilePaths,
+  journalTransactionAt,
+  journalNextTransaction,
+  journalPrevTransaction,
   journalPostings,
   -- * Standard account types
   journalBalanceSheetAccountQuery,
@@ -114,7 +117,7 @@ instance Show Journal where
 --                      ,show (jmodifiertxns j)
 --                      ,show (jperiodictxns j)
 --                      ,show $ open_timelog_entries j
---                      ,show $ historical_prices j
+--                      ,show $ jmarketprices j
 --                      ,show $ final_comment_lines j
 --                      ,show $ jContext j
 --                      ,show $ map fst $ files j
@@ -125,7 +128,7 @@ nulljournal = Journal { jmodifiertxns = []
                       , jperiodictxns = []
                       , jtxns = []
                       , open_timelog_entries = []
-                      , historical_prices = []
+                      , jmarketprices = []
                       , final_comment_lines = []
                       , jContext = nullctx
                       , files = []
@@ -134,7 +137,7 @@ nulljournal = Journal { jmodifiertxns = []
                       }
 
 nullctx :: JournalContext
-nullctx = Ctx { ctxYear = Nothing, ctxDefaultCommodityAndStyle = Nothing, ctxAccount = [], ctxAliases = [] }
+nullctx = Ctx{ctxYear=Nothing, ctxDefaultCommodityAndStyle=Nothing, ctxAccount=[], ctxAliases=[], ctxTransactionIndex=0}
 
 journalFilePath :: Journal -> FilePath
 journalFilePath = fst . mainfile
@@ -154,12 +157,26 @@ addModifierTransaction mt j = j { jmodifiertxns = mt : jmodifiertxns j }
 addPeriodicTransaction :: PeriodicTransaction -> Journal -> Journal
 addPeriodicTransaction pt j = j { jperiodictxns = pt : jperiodictxns j }
 
-addHistoricalPrice :: HistoricalPrice -> Journal -> Journal
-addHistoricalPrice h j = j { historical_prices = h : historical_prices j }
+addMarketPrice :: MarketPrice -> Journal -> Journal
+addMarketPrice h j = j { jmarketprices = h : jmarketprices j }
 
 addTimeLogEntry :: TimeLogEntry -> Journal -> Journal
 addTimeLogEntry tle j = j { open_timelog_entries = tle : open_timelog_entries j }
 
+-- | Get the transaction with this index (its 1-based position in the input stream), if any.
+journalTransactionAt :: Journal -> Integer -> Maybe Transaction
+journalTransactionAt Journal{jtxns=ts} i =
+  -- it's probably ts !! (i+1), but we won't assume
+  headMay [t | t <- ts, tindex t == i]
+
+-- | Get the transaction that appeared immediately after this one in the input stream, if any.
+journalNextTransaction :: Journal -> Transaction -> Maybe Transaction
+journalNextTransaction j t = journalTransactionAt j (tindex t + 1)
+  
+-- | Get the transaction that appeared immediately before this one in the input stream, if any.
+journalPrevTransaction :: Journal -> Transaction -> Maybe Transaction
+journalPrevTransaction j t = journalTransactionAt j (tindex t - 1)
+  
 -- | Unique transaction descriptions used in this journal.
 journalDescriptions :: Journal -> [String]
 journalDescriptions = nub . sort . map tdescription . jtxns
@@ -411,7 +428,7 @@ journalFinalise tclock tlocal path txt ctx assrt j@Journal{files=fs} = do
      , jtxns=reverse $ jtxns j -- NOTE: see addTransaction
      , jmodifiertxns=reverse $ jmodifiertxns j -- NOTE: see addModifierTransaction
      , jperiodictxns=reverse $ jperiodictxns j -- NOTE: see addPeriodicTransaction
-     , historical_prices=reverse $ historical_prices j -- NOTE: see addHistoricalPrice
+     , jmarketprices=reverse $ jmarketprices j -- NOTE: see addMarketPrice
      , open_timelog_entries=reverse $ open_timelog_entries j -- NOTE: see addTimeLogEntry
      })
   >>= if assrt then journalCheckBalanceAssertions else return
@@ -487,17 +504,19 @@ journalBalanceTransactions j@Journal{jtxns=ts, jcommoditystyles=ss} =
                                     Left e    -> Left e
       where balance = balanceTransaction (Just ss)
 
--- | Convert all the journal's posting amounts (not price amounts) to
--- their canonical display settings. Ie, all amounts in a given
--- commodity will use (a) the display settings of the first, and (b)
--- the greatest precision, of the posting amounts in that commodity.
+-- | Convert all the journal's posting amounts (and historical price
+-- amounts, but currently not transaction price amounts) to their
+-- canonical display settings. Ie, all amounts in a given commodity
+-- will use (a) the display settings of the first, and (b) the
+-- greatest precision, of the posting amounts in that commodity.
 journalCanonicaliseAmounts :: Journal -> Journal
-journalCanonicaliseAmounts j@Journal{jtxns=ts} = j''
+journalCanonicaliseAmounts j@Journal{jtxns=ts, jmarketprices=mps} = j''
     where
-      j'' = j'{jtxns=map fixtransaction ts}
+      j'' = j'{jtxns=map fixtransaction ts, jmarketprices=map fixmarketprice mps}
       j' = j{jcommoditystyles = canonicalStyles $ dbg8 "journalAmounts" $ journalAmounts j}
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
+      fixmarketprice mp@MarketPrice{mpamount=a} = mp{mpamount=fixamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
       fixamount a@Amount{acommodity=c} = a{astyle=journalCommodityStyle j' c}
 
@@ -528,8 +547,8 @@ journalCommodityStyle :: Journal -> Commodity -> AmountStyle
 journalCommodityStyle j c = M.findWithDefault amountstyle c $ jcommoditystyles j
 
 -- -- | Apply this journal's historical price records to unpriced amounts where possible.
--- journalApplyHistoricalPrices :: Journal -> Journal
--- journalApplyHistoricalPrices j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
+-- journalApplyMarketPrices :: Journal -> Journal
+-- journalApplyMarketPrices j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
 --     where
 --       fixtransaction t@Transaction{tdate=d, tpostings=ps} = t{tpostings=map fixposting ps}
 --        where
@@ -537,14 +556,14 @@ journalCommodityStyle j c = M.findWithDefault amountstyle c $ jcommoditystyles j
 --         fixmixedamount (Mixed as) = Mixed $ map fixamount as
 --         fixamount = fixprice
 --         fixprice a@Amount{price=Just _} = a
---         fixprice a@Amount{commodity=c} = a{price=maybe Nothing (Just . UnitPrice) $ journalHistoricalPriceFor j d c}
+--         fixprice a@Amount{commodity=c} = a{price=maybe Nothing (Just . UnitPrice) $ journalMarketPriceFor j d c}
 
 -- -- | Get the price for a commodity on the specified day from the price database, if known.
 -- -- Does only one lookup step, ie will not look up the price of a price.
--- journalHistoricalPriceFor :: Journal -> Day -> Commodity -> Maybe MixedAmount
--- journalHistoricalPriceFor j d Commodity{symbol=s} = do
---   let ps = reverse $ filter ((<= d).hdate) $ filter ((s==).hsymbol) $ sortBy (comparing hdate) $ historical_prices j
---   case ps of (HistoricalPrice{hamount=a}:_) -> Just a
+-- journalMarketPriceFor :: Journal -> Day -> Commodity -> Maybe MixedAmount
+-- journalMarketPriceFor j d Commodity{symbol=s} = do
+--   let ps = reverse $ filter ((<= d).mpdate) $ filter ((s==).hsymbol) $ sortBy (comparing mpdate) $ jmarketprices j
+--   case ps of (MarketPrice{mpamount=a}:_) -> Just a
 --              _ -> Nothing
 
 -- | Close any open timelog sessions in this journal using the provided current time.
@@ -581,13 +600,23 @@ journalConvertAmountsToCost j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
 --               Just (UnitPrice ma)  -> c:(concatMap amountCommodities $ amounts ma)
 --               Just (TotalPrice ma) -> c:(concatMap amountCommodities $ amounts ma)
 
--- | Get all this journal's (mixed) amounts, in the order parsed.
-journalMixedAmounts :: Journal -> [MixedAmount]
-journalMixedAmounts = map pamount . journalPostings
-
--- | Get all this journal's component amounts, roughly in the order parsed.
+-- | Get an ordered list of the amounts in this journal which will
+-- influence amount style canonicalisation. These are:
+--
+-- * amounts in market price directives (in parse order)
+-- * amounts in postings (in parse order)
+--
+-- Amounts in default commodity directives also influence
+-- canonicalisation, but earlier, as amounts are parsed.
+-- Amounts in posting prices are not used for canonicalisation.
+--
 journalAmounts :: Journal -> [Amount]
-journalAmounts = concatMap flatten . journalMixedAmounts where flatten (Mixed as) = as
+journalAmounts j =
+  concat
+   [map mpamount $ jmarketprices j
+   ,concatMap flatten $ map pamount $ journalPostings j
+   ]
+  where flatten (Mixed as) = as
 
 -- | The fully specified date span enclosing the dates (primary or secondary)
 -- of all this journal's transactions and postings, or DateSpan Nothing Nothing
@@ -671,6 +700,7 @@ Right samplejournal = journalBalanceTransactions $
          nulljournal
          {jtxns = [
            txnTieKnot $ Transaction {
+             tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/01/01",
              tdate2=Nothing,
@@ -687,6 +717,7 @@ Right samplejournal = journalBalanceTransactions $
            }
           ,
            txnTieKnot $ Transaction {
+             tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/01",
              tdate2=Nothing,
@@ -703,6 +734,7 @@ Right samplejournal = journalBalanceTransactions $
            }
           ,
            txnTieKnot $ Transaction {
+             tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/02",
              tdate2=Nothing,
@@ -719,6 +751,7 @@ Right samplejournal = journalBalanceTransactions $
            }
           ,
            txnTieKnot $ Transaction {
+             tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/03",
              tdate2=Nothing,
@@ -735,6 +768,7 @@ Right samplejournal = journalBalanceTransactions $
            }
           ,
            txnTieKnot $ Transaction {
+             tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/12/31",
              tdate2=Nothing,
