@@ -37,6 +37,7 @@ module Hledger.Data.Posting (
   joinAccountNames,
   concatAccountNames,
   accountNameApplyAliases,
+  accountNameApplyAliasesMemo,
   -- * arithmetic
   sumPostings,
   -- * rendering
@@ -48,11 +49,11 @@ module Hledger.Data.Posting (
 where
 import Data.List
 import Data.Maybe
+import Data.MemoUgly (memo)
 import Data.Ord
 import Data.Time.Calendar
 import Safe
 import Test.HUnit
-import Text.Printf
 
 import Hledger.Utils
 import Hledger.Data.Types
@@ -88,12 +89,12 @@ showPosting p@Posting{paccount=a,pamount=amt,ptype=t} =
     where
       ledger3ishlayout = False
       acctnamewidth = if ledger3ishlayout then 25 else 22
-      showaccountname = printf ("%-"++(show acctnamewidth)++"s") . bracket . elideAccountName width
+      showaccountname = fitString (Just acctnamewidth) Nothing False False . bracket . elideAccountName width
       (bracket,width) = case t of
                           BalancedVirtualPosting -> (\s -> "["++s++"]", acctnamewidth-2)
                           VirtualPosting -> (\s -> "("++s++")", acctnamewidth-2)
                           _ -> (id,acctnamewidth)
-      showamount = padleft 12 . showMixedAmount
+      showamount = padLeftWide 12 . showMixedAmount
 
 
 showComment :: String -> String
@@ -230,6 +231,10 @@ accountNameApplyAliases aliases a = accountNameWithPostingType atype aname'
              aname
              aliases
 
+-- | Memoising version of accountNameApplyAliases, maybe overkill.
+accountNameApplyAliasesMemo :: [AccountAlias] -> AccountName -> AccountName
+accountNameApplyAliasesMemo aliases = memo (accountNameApplyAliases aliases)
+
 -- aliasMatches :: AccountAlias -> AccountName -> Bool
 -- aliasMatches (BasicAlias old _) a = old `isAccountNamePrefixOf` a
 -- aliasMatches (RegexAlias re  _) a = regexMatchesCI re a
@@ -238,7 +243,7 @@ aliasReplace :: AccountAlias -> AccountName -> AccountName
 aliasReplace (BasicAlias old new) a
   | old `isAccountNamePrefixOf` a || old == a = new ++ drop (length old) a
   | otherwise = a
-aliasReplace (RegexAlias re repl) a = regexReplaceCI re repl a
+aliasReplace (RegexAlias re repl) a = regexReplaceCIMemo re repl a
 
 
 tests_Hledger_Data_Posting = TestList [
