@@ -30,6 +30,7 @@ module Hledger.Data.Transaction (
   -- * rendering
   showTransaction,
   showTransactionUnelided,
+  showTransactionUnelidedOneLineAmounts,
   -- * misc.
   tests_Hledger_Data_Transaction
 )
@@ -40,7 +41,6 @@ import Data.Time.Calendar
 import Test.HUnit
 import Text.Printf
 import qualified Data.Map as Map
-import Text.Parsec.Pos
 
 import Hledger.Utils
 import Hledger.Data.Types
@@ -56,11 +56,12 @@ instance Show ModifierTransaction where
 instance Show PeriodicTransaction where
     show t = "~ " ++ ptperiodicexpr t ++ "\n" ++ unlines (map show (ptpostings t))
 
-nullsourcepos :: SourcePos
-nullsourcepos = initialPos ""
+nullsourcepos :: GenericSourcePos
+nullsourcepos = GenericSourcePos "" 1 1
 
 nulltransaction :: Transaction
 nulltransaction = Transaction {
+                    tindex=0,
                     tsourcepos=nullsourcepos,
                     tdate=nulldate,
                     tdate2=Nothing,
@@ -90,10 +91,10 @@ pcommentwidth = no limit -- 22
 @
 -}
 showTransaction :: Transaction -> String
-showTransaction = showTransaction' True
+showTransaction = showTransactionHelper True False
 
 showTransactionUnelided :: Transaction -> String
-showTransactionUnelided = showTransaction' False
+showTransactionUnelided = showTransactionHelper False False
 
 tests_showTransactionUnelided = [
    "showTransactionUnelided" ~: do
@@ -128,18 +129,19 @@ tests_showTransactionUnelided = [
       ]
  ]
 
+showTransactionUnelidedOneLineAmounts :: Transaction -> String
+showTransactionUnelidedOneLineAmounts = showTransactionHelper False True
+
 -- cf showPosting
-showTransaction' :: Bool -> Transaction -> String
-showTransaction' elide t =
+showTransactionHelper :: Bool -> Bool -> Transaction -> String
+showTransactionHelper elide onelineamounts t =
     unlines $ [descriptionline]
               ++ newlinecomments
-              ++ (postingsAsLines elide t (tpostings t))
+              ++ (postingsAsLines elide onelineamounts t (tpostings t))
               ++ [""]
     where
       descriptionline = rstrip $ concat [date, status, code, desc, samelinecomment]
-      date = showdate (tdate t) ++ maybe "" showedate (tdate2 t)
-      showdate = printf "%-10s" . showDate
-      showedate = printf "=%s" . showdate
+      date = showDate (tdate t) ++ maybe "" (("="++) . showDate) (tdate2 t)
       status | tstatus t == Cleared = " *"
              | tstatus t == Pending = " !"
              | otherwise            = ""
@@ -167,33 +169,41 @@ renderCommentLines s  = case lines s of ("":ls) -> "":map commentprefix ls
 --       ls = lines s
 --       prefix = indent . (";"++)
 
-postingsAsLines :: Bool -> Transaction -> [Posting] -> [String]
-postingsAsLines elide t ps
+postingsAsLines :: Bool -> Bool -> Transaction -> [Posting] -> [String]
+postingsAsLines elide onelineamounts t ps
     | elide && length ps > 1 && isTransactionBalanced Nothing t -- imprecise balanced check
-       = (concatMap (postingAsLines False ps) $ init ps) ++ postingAsLines True ps (last ps)
-    | otherwise = concatMap (postingAsLines False ps) ps
+       = (concatMap (postingAsLines False onelineamounts ps) $ init ps) ++ postingAsLines True onelineamounts ps (last ps)
+    | otherwise = concatMap (postingAsLines False onelineamounts ps) ps
 
-postingAsLines :: Bool -> [Posting] -> Posting -> [String]
-postingAsLines elideamount ps p =
+postingAsLines :: Bool -> Bool -> [Posting] -> Posting -> [String]
+postingAsLines elideamount onelineamounts ps p =
     postinglines
     ++ newlinecomments
   where
-    postinglines = map rstrip $ lines $ concatTopPadded [showacct p, "  ", amount, samelinecomment]
-    amount = if elideamount then "" else showamt (pamount p)
+    postinglines = map rstrip $ lines $ concatTopPadded [account, "  ", amount, samelinecomment]
+
+    account =
+      indent $
+        showstatus p ++ fitString (Just acctwidth) Nothing False True (showAccountName Nothing (ptype p) (paccount p))
+        where
+          showstatus p = if pstatus p == Cleared then "* " else ""
+          acctwidth = maximum $ map (strWidth . paccount) ps
+
+    -- currently prices are considered part of the amount string when right-aligning amounts
+    amount
+      | elideamount    = ""
+      | onelineamounts = fitString (Just amtwidth) Nothing False False $ showMixedAmountOneLine $ pamount p
+      | otherwise      = fitStringMulti (Just amtwidth) Nothing False False $ showMixedAmount $ pamount p
+      where
+        amtwidth = maximum $ 12 : map (strWidth . showMixedAmount . pamount) ps  -- min. 12 for backwards compatibility
+
     (samelinecomment, newlinecomments) =
       case renderCommentLines (pcomment p) of []   -> ("",[])
                                               c:cs -> (c,cs)
-    showacct p =
-      indent $ showstatus p ++ printf (printf "%%-%ds" w) (showAccountName Nothing (ptype p) (paccount p))
-        where
-          showstatus p = if pstatus p == Cleared then "* " else ""
-          w = maximum $ map (length . paccount) ps
-    showamt =
-        padleft 12 . showMixedAmount
 
 tests_postingAsLines = [
    "postingAsLines" ~: do
-    let p `gives` ls = assertEqual "" ls (postingAsLines False [p] p)
+    let p `gives` ls = assertEqual "" ls (postingAsLines False False [p] p)
     posting `gives` ["                 0"]
     posting{
       pstatus=Cleared,
@@ -410,7 +420,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,"    assets:checking"
         ,""
         ])
-       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
+       (let t = Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
                 [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
                 ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
@@ -424,7 +434,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,"    assets:checking               $-47.18"
         ,""
         ])
-       (let t = Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
+       (let t = Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
                 [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18], ptransaction=Just t}
                 ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.18)], ptransaction=Just t}
                 ] ""
@@ -440,7 +450,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
          ,posting{paccount="assets:checking", pamount=Mixed [usd (-47.19)]}
          ] ""))
@@ -453,7 +463,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=Mixed [usd 47.18]}
          ] ""))
 
@@ -465,7 +475,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "coopportunity" "" []
          [posting{paccount="expenses:food:groceries", pamount=missingmixedamt}
          ] ""))
 
@@ -478,7 +488,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
         ,""
         ])
        (showTransaction
-        (txnTieKnot $ Transaction nullsourcepos (parsedate "2010/01/01") Nothing Uncleared "" "x" "" []
+        (txnTieKnot $ Transaction 0 nullsourcepos (parsedate "2010/01/01") Nothing Uncleared "" "x" "" []
          [posting{paccount="a", pamount=Mixed [num 1 `at` (usd 2 `withPrecision` 0)]}
          ,posting{paccount="b", pamount= missingmixedamt}
          ] ""))
@@ -486,19 +496,19 @@ tests_Hledger_Data_Transaction = TestList $ concat [
   ,"balanceTransaction" ~: do
      assertBool "detect unbalanced entry, sign error"
                     (isLeft $ balanceTransaction Nothing
-                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
+                           (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
                             [posting{paccount="a", pamount=Mixed [usd 1]}
                             ,posting{paccount="b", pamount=Mixed [usd 1]}
                             ] ""))
 
      assertBool "detect unbalanced entry, multiple missing amounts"
                     (isLeft $ balanceTransaction Nothing
-                           (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
+                           (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "test" "" []
                             [posting{paccount="a", pamount=missingmixedamt}
                             ,posting{paccount="b", pamount=missingmixedamt}
                             ] ""))
 
-     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "" "" []
+     let e = balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2007/01/28") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1]}
                            ,posting{paccount="b", pamount=missingmixedamt}
                            ] "")
@@ -509,7 +519,7 @@ tests_Hledger_Data_Transaction = TestList $ concat [
                         Right e' -> (pamount $ last $ tpostings e')
                         Left _ -> error' "should not happen")
 
-     let e = balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
+     let e = balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1.35]}
                            ,posting{paccount="b", pamount=Mixed [eur (-1)]}
                            ] "")
@@ -521,49 +531,49 @@ tests_Hledger_Data_Transaction = TestList $ concat [
                         Left _ -> error' "should not happen")
 
      assertBool "balanceTransaction balances based on cost if there are unit prices" (isRight $
-       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
+       balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1 `at` eur 2]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) `at` eur 1]}
                            ] ""))
 
      assertBool "balanceTransaction balances based on cost if there are total prices" (isRight $
-       balanceTransaction Nothing (Transaction nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
+       balanceTransaction Nothing (Transaction 0 nullsourcepos (parsedate "2011/01/01") Nothing Uncleared "" "" "" []
                            [posting{paccount="a", pamount=Mixed [usd 1    @@ eur 1]}
                            ,posting{paccount="a", pamount=Mixed [usd (-2) @@ eur 1]}
                            ] ""))
 
   ,"isTransactionBalanced" ~: do
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ] ""
      assertBool "detect balanced" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.01)], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ] ""
      assertBool "detect unbalanced, one posting" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 0], ptransaction=Just t}
              ] ""
      assertBool "one zero posting is considered balanced for now" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=VirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "virtual postings don't need to balance" (isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
              ] ""
      assertBool "balanced virtual postings need to balance among themselves" (not $ isTransactionBalanced Nothing t)
-     let t = Transaction nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
+     let t = Transaction 0 nullsourcepos (parsedate "2009/01/01") Nothing Uncleared "" "a" "" []
              [posting{paccount="b", pamount=Mixed [usd 1.00], ptransaction=Just t}
              ,posting{paccount="c", pamount=Mixed [usd (-1.00)], ptransaction=Just t}
              ,posting{paccount="d", pamount=Mixed [usd 100], ptype=BalancedVirtualPosting, ptransaction=Just t}
