@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards, DeriveDataTypeable #-}
+{-# LANGUAGE CPP, RecordWildCards, DeriveDataTypeable #-}
 {-|
 
 Options common to most hledger reports.
@@ -12,6 +12,7 @@ module Hledger.Reports.ReportOptions (
   FormatStr,
   defreportopts,
   rawOptsToReportOpts,
+  checkReportOpts,
   flat_,
   tree_,
   dateSpanFromOpts,
@@ -30,6 +31,9 @@ module Hledger.Reports.ReportOptions (
 where
 
 import Data.Data (Data)
+#if !MIN_VERSION_base(4,8,0)
+import Data.Functor.Compat ((<$>))
+#endif
 import Data.Typeable (Typeable)
 import Data.Time.Calendar
 import System.Console.CmdArgs.Default  -- some additional default stuff
@@ -88,6 +92,7 @@ data ReportOpts = ReportOpts {
     ,drop_           :: Int
     ,row_total_      :: Bool
     ,no_total_       :: Bool
+    ,value_          :: Bool
  } deriving (Show, Data, Typeable)
 
 instance Default ReportOpts where def = defreportopts
@@ -121,9 +126,10 @@ defreportopts = ReportOpts
     def
     def
     def
+    def
 
 rawOptsToReportOpts :: RawOpts -> IO ReportOpts
-rawOptsToReportOpts rawopts = do
+rawOptsToReportOpts rawopts = checkReportOpts <$> do
   d <- getCurrentDay
   return defreportopts{
      begin_       = maybesmartdateopt d "begin" rawopts
@@ -144,7 +150,7 @@ rawOptsToReportOpts rawopts = do
     ,monthly_     = boolopt "monthly" rawopts
     ,quarterly_   = boolopt "quarterly" rawopts
     ,yearly_      = boolopt "yearly" rawopts
-    ,format_      = maybestringopt "format" rawopts
+    ,format_      = maybestringopt "format" rawopts -- XXX move to CliOpts or move validation from Cli.CliOptions to here
     ,query_       = unwords $ listofstringopt "args" rawopts -- doesn't handle an arg like "" right
     ,average_     = boolopt "average" rawopts
     ,related_     = boolopt "related" rawopts
@@ -153,8 +159,17 @@ rawOptsToReportOpts rawopts = do
     ,drop_        = intopt "drop" rawopts
     ,row_total_   = boolopt "row-total" rawopts
     ,no_total_    = boolopt "no-total" rawopts
+    ,value_       = boolopt "value" rawopts
     }
 
+-- | Do extra validation of opts, raising an error if there is trouble.
+checkReportOpts :: ReportOpts -> ReportOpts
+checkReportOpts ropts@ReportOpts{..} =
+  either optserror (const ropts) $ do
+    case depth_ of
+      Just d | d < 0 -> Left "--depth should have a positive number"
+      _              -> Right ()
+
 accountlistmodeopt :: RawOpts -> AccountListMode
 accountlistmodeopt rawopts =
   case reverse $ filter (`elem` ["tree","flat"]) $ map fst rawopts of
